.include "cpctelera.h.s"
.include "entity.h.s"
.include "template.h.s"
.include "animation.h.s"
.include "patrol.h.s"
.include "level.h.s"
.include "score.h.s"
.include "game.h.s"
.include "text.h.s"

.include "../systems/render.h.s" 
.include "../systems/physics.h.s"
.include "../systems/ai.h.s"
.include "../systems/colission.h.s"
.include "../sprites/screenStart_z.h.s"
.include "../sprites/tutorial_z.h.s"

.globl cpct_waitVSYNC_asm
.globl cpct_setVideoMode_asm

.globl cpct_scanKeyboard_if_asm
.globl cpct_isKeyPressed_asm

.globl cpct_zx7b_decrunch_s_asm
.globl cpct_drawSprite_asm

.globl _myDrawCharInnerM0
.globl _myDrawStringM0
.globl _mySetDrawCharM0

.globl cpct_akp_musicInit_asm
.globl cpct_akp_musicPlay_asm
.globl _songDemo
.globl _sounds
.globl cpct_akp_SFXInit_asm
.globl cpct_scanKeyboard_asm
.globl cpct_isAnyKeyPressed_asm



lvlActual:: .db 0   ;; indice del array de niveles
numEnemies:: .db 0  ;; numero de enemigos que faltan por matar para poder pasar de nivel (contador)

contInterruptions: .db 06
int_handler::

  push af
  push bc
  push de
  push hl
  
  ld a, (contInterruptions)
  dec a
  jr nz, notZero
    call cpct_akp_musicPlay_asm
    call cpct_scanKeyboard_if_asm
    ld a, #6          ;cuando es cero reseteamos el contador
  notZero:
  ld (contInterruptions),a

  pop hl
  pop de
  pop bc
  pop af

  ei
reti

set_int_handler::

  call cpct_waitVSYNC_asm
  halt
  halt
  call cpct_waitVSYNC_asm     ;;esto se hace para asegurarnos que las interrupciones siempre empiezan al incio del vsync (abajo)

  ld hl, #0x38
  ld (hl), #0xC3
  inc hl
  ld (hl), #<int_handler
  inc hl
  ld (hl), #>int_handler
  inc hl
  ld (hl), #0xC9
ret

gameInit::

  halt        ;para asegurarnos que el cpct_setVideoMode_asm comienza con una interrupcion
  ld c,#0
  call cpct_setVideoMode_asm

  ld hl, #_g_palette
  ld de, #16
  call cpct_setPalette_asm

  cpctm_setBorder_asm HW_BLACK
  
  ;;iniciamos color de las letras
  ld hl, #0x000F
  call _mySetDrawCharM0

  ;;inicializar puntos
  ld hl, #0
  ld (score), hl

  ld a, #0
  ld (lvlActual), a
  ld (win), a
  ld (waste), a
  ld (contEnemyShot),a  ;disparos de los enemigos

  ld a, #0x50
  ld (actualHP), a

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;PARA REDUCIR EL COOLDOWN DEL DISPARO
  ;;inicar cooldown del arma inicial
  ld a, #coolDownShotInit
  ld (coolDownShot), a

  ;nivel que se muestra por pantalla
  ld a, #1
  ld (levelScreen),a

  ;inicio
  call startScreen
ret

counter: .db 0
paint=15
gamePlay::
  
  call set_int_handler

  ld de, #_songDemo
  call cpct_akp_musicInit_asm
  ld de,#_sounds
  call cpct_akp_SFXInit_asm


  call initRanking

  mainLoop:

  call gameInit
  call loadLevel
    
    call renderEntities

    gameLoop:
       
      call checkLevel
      call aiUpdate
      call updatePhysics
      call updateAnimation
      call cpct_waitVSYNC_asm
      call renderEntities
      call updateCollision
      call updateAliveEntities

      ld a,(counter)
      cp #paint
      jr z,noPaint
        call showStatus
        ld a,#paint
        ld (counter),a
        jr painted
      noPaint:
      dec a
      ld (counter),a
      painted:



      ld a, (waste)
      add #0
      jp nz, endGameOver

      ld a, (win)
      add #0
      jp nz, endGameWin

      ;;restamos cooldown de golpe

      ;bajamos el contador del cooldown de daño del personaje en el caso de no ser 0
      ld ix, #_arrayEntities
      ld a, e_dmg(ix)
      cp #0
      jp z, endCooldown
        dec a
        ld e_dmg(ix),a
      endCooldown:


      call cpct_waitVSYNC_asm

    jp gameLoop

    endGameOver:
      call showGameOver

    jp mainLoop

    endGameWin:
      call showWinScreen

  jp mainLoop

ret

showStatus::
 call showHealth
      ;;pasamos puntero a score y lo mostramos
      ld hl, (score)
      ld de, #0xc08e
      call showScore
ret
levelScreen:: .db 1
showLevel::
  ;;color blanco de las letras
  ld hl, #0x0E0F
  call _mySetDrawCharM0

  ld a, (levelScreen)
  push af
  rra                 ;0001 0000
  rra                 ;0000 1000
  rra                 ;0000 0100
  rra                 ;0000 0010 con esto ya hemos sacado el numero en decimal 2 que queremos imprimir
  and #0x0F             ;; para quedarnos con los cuatro bits de atras solo
  add #48             ;; en "a" ya tenemos el caracter del 2 en ascii
  ld e, a
  ld hl, #0xc077
  call _myDrawCharInnerM0

  pop af
  and #0x0F             ;; para quedarnos con los cuatro bits de atras solo
  add #48             ;; en "a" ya tenemos el caracter del 2 en ascii
  ld e, a
  ld hl, #0xc07B
  call _myDrawCharInnerM0

ret

;;
showHealth::

    ld ix, #_arrayEntities
    ld a, e_HP(ix)             ;desplazo los bits
    rra                 ;0001 0000
    rra                 ;0000 1000
    rra                 ;0000 0100
    rra                 ;0000 0010 con esto ya hemos sacado el numero en decimal 2 que queremos imprimir
    and #0x0F             ;; para quedarnos con los cuatro bits de atras solo
    add #48             ;; en "a" ya tenemos el caracter del 2 en ascii
    ld e, a
    ld hl, #0xc059
    call _myDrawCharInnerM0

    ld ix, #_arrayEntities
    ld a, e_HP(ix)             ;desplazo los bits
    and #0x0F             ;; para quedarnos con los cuatro bits de atras solo
    add #48             ;; en "a" ya tenemos el caracter del 2 en ascii
    ld e, a
    ld hl, #0xc05D
    call _myDrawCharInnerM0

    ld a,(counterColor)
    cp #0
    jr nz,noChangeColor
      ld hl, #0x020F
      call _mySetDrawCharM0
      ret
    noChangeColor:
    ld a,(counterColor)
    dec a
    ld (counterColor),a

ret

waitSpace::
  loopStartScreen:
        ld hl, #Joy0_Fire1
        call cpct_isKeyPressed_asm
        ret nz

        ld hl, #Joy0_Fire2
        call cpct_isKeyPressed_asm
        ret nz

        ld hl, #Key_Space
        call cpct_isKeyPressed_asm
    jr z, loopStartScreen

ret

startScreen::
  ;;  dibujar fondo
    ld hl, #_screenStart_z_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm

  ;;  comprobar teclado para empezar la partida
    call waitSpace

  ;;pantalla tutorial

    ld hl, #_tutorial_z_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm

    call waitSpace

  ;;  limpiar pantalla
    ld hl, #0xC000
    ld (hl), #0
    ld de, #0xC001
    ld bc, #0x4000
    ldir

    call showLore

ret


showGameOver::

  ;;  limpiar pantalla
  ld hl, #0xC000
  ld (hl), #0
  ld de, #0xC001
  ld bc, #0x4000
  ldir

    ;; fondo negro
  ld l, #0
  ld h, #0x14
  call cpct_setPALColour_asm

  call decorateGameOver

    ;;  comprobar teclado para reiniciar la partida
    call waitSpace
    
    call resetGame 
    
ret

showWinScreen::

  ;;  limpiar pantalla
  ld hl, #0xC000
  ld (hl), #0
  ld de, #0xC001
  ld bc, #0x4000
  ldir

  ;paleta
  ld l, #0
  ld h, #0x14
  call cpct_setPALColour_asm

  call decorateWIn

  ;  comprobar teclado para reiniciar la partida
  call showInputScreen
  call createScore

  call showRanking

  call waitSpace

  call resetGame 
ret


phraseWin: .asciz "WIN!"
phraseScore: .asciz "SCORE:"
phraseYourName: .asciz "NAME:"
decorateWIn::

  ;;iniciamos color de las letras con color amarillo
  ld hl, #0x000D
  call _mySetDrawCharM0

  ;titulo win
  ld hl, #0xc110
  ld iy, #phraseWin
  call _myDrawStringM0

  ;pintamos estrella izquierda
  ld de, #0xc054
  ld c, #16                                       
  ld b, #32                  
  ld hl, #_decoration_2
  call cpct_drawSprite_asm

  ;pintamos estrella derecha
  ld de, #0xc08c
  ld c, #16                                       
  ld b, #32                  
  ld hl, #_decoration_2
  call cpct_drawSprite_asm

  ;;iniciamos color de las letras con color blanco
  ld hl, #0x000F
  call _mySetDrawCharM0

  ;score
  ld hl, #0xc2e0
  ld iy, #phraseScore
  call _myDrawStringM0

  ld hl, (score)
  ld de, #0xc2fc
  call showScore

  ;score
  ld hl, #0xc424 
  ld iy, #phraseYourName
  call _myDrawStringM0

  ;;iniciamos color de las letras con color blanco
  ld hl, #0x000F
  call _mySetDrawCharM0

  ld a, #0x5f
  ld hl, #0xc43c 
  call _myDrawCharInnerM0

  ld a, #0x5f
  ld hl, #0xc440
  call _myDrawCharInnerM0

  ld a, #0x5f
  ld hl, #0xc444
  call _myDrawCharInnerM0

  ;pintamos copa 
  ld de, #0xc60c
  ld c, #4                                       
  ld b, #10                  
  ld hl, #_treasure_0
  call cpct_drawSprite_asm

  ;pintamos copa 
  ld de, #0xc570
  ld c, #16                                       
  ld b, #32                  
  ld hl, #_decoration_0
  call cpct_drawSprite_asm

  ;pintamos copa 
  ld de, #0xc620
  ld c, #4                                       
  ld b, #10                  
  ld hl, #_treasure_0
  call cpct_drawSprite_asm

ret

phraseGameOver: .asciz "GAME  OVER"
phraseFloor: .asciz "FLOOR:"

decorateGameOver::
  ;;iniciamos color de las letras con color amarillo
  ld hl, #0x0004
  call _mySetDrawCharM0

  ;titulo win
  ld hl, #0xc104
  ld iy, #phraseGameOver
  call _myDrawStringM0

  ld hl, #0xc74c
  ld iy, #phraseSpace
  call _myDrawStringM0

  ;pintamos copa 
  ld de, #0xc051
  ld c, #16                                       
  ld b, #32                  
  ld hl, #_decoration_1
  call cpct_drawSprite_asm

  ld de, #0xc090
  ld c, #16                                       
  ld b, #32                  
  ld hl, #_decoration_1
  call cpct_drawSprite_asm

  ld hl, #0x000F
  call _mySetDrawCharM0

  ;score
  ld hl, #0xc382
  ld iy, #phraseScore
  call _myDrawStringM0

  ld hl, (score)
  ld de, #0xc39e
  call showScore


  ;FLOOR
  ld hl, #0xc422
  ld iy, #phraseFloor
  call _myDrawStringM0

  ;simbolo -
  ld a, #0x2D
  ld hl, #0xc43e
  call _myDrawCharInnerM0

  ;numero del nivel
  ld a, (levelScreen)
  push af
  rra                 ;0001 0000
  rra                 ;0000 1000
  rra                 ;0000 0100
  rra                 ;0000 0010 con esto ya hemos sacado el numero en decimal 2 que queremos imprimir
  and #0x0F             ;; para quedarnos con los cuatro bits de atras solo
  add #48             ;; en "a" ya tenemos el caracter del 2 en ascii
  ld e, a
  ld hl, #0xc442
  call _myDrawCharInnerM0

  pop af
  and #0x0F             ;; para quedarnos con los cuatro bits de atras solo
  add #48             ;; en "a" ya tenemos el caracter del 2 en ascii
  ld e, a
  ld hl, #0xc446
  call _myDrawCharInnerM0

  ;Simbolo radioactivo
  ld de, #0xc51e 
  ld c, #16                                       
  ld b, #32                  
  ld hl, #_decoration_3
  call cpct_drawSprite_asm
  

ret


resetGame::
   ld a, #0
   ld (_reservedEntities), a
   ld (enemysScreen),a
   ld (screenSpawner),a    ;LIMITADOR PANTALLA
   ld hl, #_arrayEntities
   ld (_nextEntity), hl


   ld hl, #_arrayEntities
   ld de, #_sizeArray
   ld iy, #0

   bucle:
      ld (hl), #0x0
      inc hl
      dec de

      ld a, d
      add #0
      jr nz, bucle

      ld a, e
      add #0

      jr nz, bucle

ret

showLore::
  ;; fondo negro
  ld l, #0
  ld h, #0x14
  call cpct_setPALColour_asm

  ld ix,#arrayTextLore

    ld b,(ix) ;;numero de frases
    ld a,1(ix) ;;parte pequeña
    ld l,a
    ld a,2(ix)  ;;parte grande
    ld h,a

    call centerTextVertical 

    ld hl, #0xc74c
    ld iy, #phraseSpace
    call _myDrawStringM0

    call waitSpace

  ld hl, #0xC000
    ld (hl), #0
    ld de, #0xC001
    ld bc, #0x4000
    ldir
    
  ld ix,#arrayTextLore
  ld bc, #3
  add ix, bc

  ld b,(ix) ;;numero de frases
    ld a,1(ix) ;;parte pequeña
    ld l,a
    ld a,2(ix)  ;;parte grande
    ld h,a

    call centerTextVertical

    ld hl, #0xc74c
    ld iy, #phraseSpace
    call _myDrawStringM0 


  espera:
   call cpct_scanKeyboard_asm
   call cpct_isAnyKeyPressed_asm

   jr nz, espera

    call waitSpace


ret